package ch.metas.soa.prototyping.jboss.marshalling.ejb;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import javax.ejb.EJB;
import javax.ejb.Stateless;

import org.hibernate.proxy.HibernateProxy;
import org.hibernate.proxy.LazyInitializer;

import ch.metas.soa.prototyping.jboss.marshalling.ifc.SecuredDemoService;
import ch.metas.soa.prototyping.jboss.marshalling.models.Book;
import ch.metas.soa.prototyping.jboss.marshalling.models.BookShelf;

@Stateless
@EJB(name = "java:global/ejb2/SecureDemoService", beanInterface = SecuredDemoService.class)
public class DemoService2 implements SecuredDemoService {

	@EJB
	DAO m_dao;

	/* (non-Javadoc)
	 * @see ch.metas.soa.prototyping.jboss.marshalling.ejb.DemoService#getBookShelf()
	 */
	@Override
	public BookShelf getBookShelf() {
		BookShelf shelf = m_dao.findFirstBookShelfById(1);
		shelf = detachFromHibernate(shelf);
		return shelf;
	}

	@Override
	public Book getFirstBook() {
		Book book = m_dao.findFirstBookById(1);
		book = detachFromHibernate(book);
		return book;
	}

	protected <T> T detachFromHibernate(T result) {
		// dummy workaround becaue jboss marshalling actually doesn't use correct serialization
		// mechanism which will detach the hibernate proxies

		if (result == null) {
			return null;
		}

		LazyInitializer initializer = null;
		if (result instanceof LazyInitializer) {
			initializer = (LazyInitializer)result;
		}
		else if (result instanceof HibernateProxy) {
			initializer = ((HibernateProxy)result).getHibernateLazyInitializer();
		}

		if (initializer != null && initializer.isUninitialized()) {
			return null;
		}


		if (result instanceof Iterable<?>) {
			Iterable col = (Iterable)result;
			for (Object entry : col) {
				detachFromHibernate(entry);
				// howto replace if null within iteratable?
			}
		}
		else {
			Field[] fields = result.getClass().getDeclaredFields();
			for (Field field : fields) {
				if (Modifier.isFinal(field.getModifiers())) {
					continue;
				}
				field.setAccessible(true);
				try {
					Object object = field.get(result);
					Object detachedObject = detachFromHibernate(object);
					if (object != detachedObject) {
						field.set(result, detachedObject);
					}
				}
				catch (IllegalArgumentException e) {
				}
				catch (IllegalAccessException e) {
				}
			}
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see ch.metas.soa.prototyping.jboss.marshalling.ejb.DemoService#initialize()
	 */
	@Override
	public void initialize() {
		BookShelf bookShelf = getBookShelf();
		if (bookShelf != null) {
			return;
		}

		bookShelf = new BookShelf();
		bookShelf.setId(1);
		bookShelf = m_dao.save(bookShelf);

		// create books
		Book book1 = new Book();
		book1.setId(1);
		book1.setName("Book1");
		book1.setBookShelf(bookShelf);
		book1 = m_dao.save(book1);

		Book book2 = new Book();
		book2.setId(2);
		book2.setName("Book2");
		book2.setBookShelf(bookShelf);
		book2 = m_dao.save(book2);
	}
}
